@@ -28,15 +28,16 @@ HELP() {
   echo
   echo "Run build tests"
   echo "Options:"
-  echo " -k, --no-krazy        Don't run any Krazy tests"
-  echo " -c, --no-cppcheck     Don't run any cppcheck tests"
-  echo " -t, --no-tidy         Don't run any clang-tidy tests"
-  echo " -b, --no-scan         Don't run any scan-build tests"
-  echo " -s, --no-splint       Don't run any splint tests"
-  echo " -l, --no-clang-build  Don't run any clang-build tests"
-  echo " -g, --no-gcc-build    Don't run any gcc-build tests"
-  echo " -a, --no-asan-build   Don't run any ASAN-build tests"
-  echo " -d, --no-tsan-build   Don't run any TSAN-build tests"
+  echo " -m, --no-cmake-compat  Don't require CMake version compatibility"
+  echo " -k, --no-krazy         Don't run any Krazy tests"
+  echo " -c, --no-cppcheck      Don't run any cppcheck tests"
+  echo " -t, --no-tidy          Don't run any clang-tidy tests"
+  echo " -b, --no-scan          Don't run any scan-build tests"
+  echo " -s, --no-splint        Don't run any splint tests"
+  echo " -l, --no-clang-build   Don't run any clang-build tests"
+  echo " -g, --no-gcc-build     Don't run any gcc-build tests"
+  echo " -a, --no-asan-build    Don't run any ASAN-build tests"
+  echo " -d, --no-tsan-build    Don't run any TSAN-build tests"
   echo
 }
 
@@ -85,7 +86,12 @@ CHECK_WARNINGS() {
   then
     echo "EXITING. $w warnings encountered"
     echo
-    cat $1 | grep "$2" | grep -v "$3" | sort | uniq
+    if ( test -n "$3")
+    then
+      cat $1 | grep "$2" | grep -v "$3" | sort | uniq
+    else
+      cat $1 | grep "$2" | sort | uniq
+    fi
     exit 1
   fi
 }
@@ -94,7 +100,7 @@ CHECK_WARNINGS() {
 # print warnings found in the compile-stage output
 # $1 = file with the compile-stage output
 COMPILE_WARNINGS() {
-  whitelist='\(no[[:space:]]link[[:space:]]for:\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|G_ADD_PRIVATE\|g_type_class_add_private.*is[[:space:]]deprecated\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|U_PLATFORM_HAS_WINUWP_API\|const[[:space:]]DBT\)'
+  whitelist='\(i-cal-object\.c\|no[[:space:]]link[[:space:]]for:\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|G_ADD_PRIVATE\|g_type_class_add_private.*is[[:space:]]deprecated\|g-ir-scanner:\|/gobject/gtype\.h\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|U_PLATFORM_HAS_WINUWP_API\|const[[:space:]]DBT\|-Llib\)'
   CHECK_WARNINGS $1 "warning:" "$whitelist"
 }
 
@@ -109,7 +115,8 @@ CPPCHECK_WARNINGS() {
 # print warnings find in the clang-tidy output
 # $1 = file with the clang-tidy output
 TIDY_WARNINGS() {
-  whitelist='\(Value[[:space:]]descriptions\|unused[[:space:]]declarations\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|modernize-\|cppcoreguidelines-pro-type-const-cast\|cppcoreguidelines-pro-type-reinterpret-cast\|cppcoreguidelines-owning-memory\|fuchsia.*\|hicpp-use-auto\|hicpp-no-malloc\|hicpp-use-nullptr\|hicpp-exception-baseclass\|hicpp-vararg\|cppcoreguidelines-pro-type-vararg\|cppcoreguidelines-pro-bounds-pointer-arithmetic\|google-build-using-namespace\|llvm-include-order\|hicpp-use-equals-default\|cppcoreguidelines-no-malloc\|g_type_class_add_private.*is[[:space:]]deprecated\)'
+  #whitelist='\(Value[[:space:]]descriptions\|unused[[:space:]]declarations\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|modernize-\|cppcoreguidelines-pro-type-const-cast\|cppcoreguidelines-pro-type-vararg\|cppcoreguidelines-pro-type-reinterpret-cast\|cppcoreguidelines-owning-memory\|fuchsia.*\|hicpp-use-auto\|hicpp-no-malloc\|hicpp-use-nullptr\|hicpp-exception-baseclass\|hicpp-vararg\|cppcoreguidelines-pro-type-vararg\|cppcoreguidelines-pro-bounds-pointer-arithmetic\|google-build-using-namespace\|llvm-include-order\|hicpp-use-equals-default\|cppcoreguidelines-no-malloc\|g_type_class_add_private.*is[[:space:]]deprecated\)'
+  whitelist='\(no[[:space:]]link[[:space:]]for:\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|G_ADD_PRIVATE\|g_type_class_add_private.*is[[:space:]]deprecated\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\)'
   CHECK_WARNINGS $1 "warning:" "$whitelist"
 }
 
@@ -117,8 +124,8 @@ TIDY_WARNINGS() {
 # print warnings found in the scan-build output
 # $1 = file with the scan-build output
 SCAN_WARNINGS() {
-  whitelist='\(no[[:space:]]link[[:space:]]for:\|g_type_class_add_private.*is[[:space:]]deprecated\|/vcc\.c\|/vobject\.c\|/icalsslexer\.c\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|icalerror.*Dereference[[:space:]]of[[:space:]]null[[:space:]]pointer\)'
-  CHECK_WARNINGS $1 "warning:" "$whitelist"
+  whitelist='\(no[[:space:]]link[[:space:]]for:\|g_type_class_add_private.*is[[:space:]]deprecated\|libical-glib-scan\.c\|/i-cal-object\.c\|/vcc\.c\|/vobject\.c\|/icalsslexer\.c\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|icalerror.*Dereference[[:space:]]of[[:space:]]null[[:space:]]pointer\|G_ADD_PRIVATE\)'
+  CHECK_WARNINGS $1 "warning:" $whitelist
 }
 
 #function CONFIGURE:
@@ -149,6 +156,13 @@ BUILD() {
   CONFIGURE "$1" "$2"
   make 2>&1 | tee make.out || exit 1
   COMPILE_WARNINGS make.out
+
+  if (test "`uname -s`" = "Darwin")
+  then
+    export DYLD_LIBRARY_PATH=$BDIR/lib
+  else
+    export LD_LIBRARY_PATH=$BDIR/lib
+  fi
   make test 2>&1 | tee make-test.out || exit 1
   CLEAN
 }
@@ -378,7 +392,7 @@ CLANGTIDY() {
   echo "===== START CLANG-TIDY: $1 ====="
   cd $TOP
   SET_CLANG
-  CONFIGURE "$1-tidy" "$2 -DCMAKE_CXX_CLANG_TIDY=clang-tidy;-checks=*"
+  CONFIGURE "$1-tidy" "$2 -DCMAKE_CXX_CLANG_TIDY=clang-tidy"
   cmake --build . 2>&1 | tee make-tidy.out || exit 1
   TIDY_WARNINGS make-tidy.out
   CLEAN
@@ -436,12 +450,13 @@ KRAZY() {
 
 ##### END FUNCTIONS #####
 
-#TEMP=`getopt -o hkctbslgad --long help,no-krazy,no-cppcheck,no-tidy,no-scan,no-splint,no-clang-build,no-gcc-build,no-asan-build,no-tsan-build -- "$@"`
-TEMP=`getopt hkctbslgad $*`
+#TEMP=`getopt -o hmkctbslgad --long help,no-cmake-compat,no-krazy,no-cppcheck,no-tidy,no-scan,no-splint,no-clang-build,no-gcc-build,no-asan-build,no-tsan-build -- "$@"`
+TEMP=`getopt hmkctbslgad $*`
 if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi
 # Note the quotes around `$TEMP': they are essential!
 eval set -- "$TEMP"
 
+cmakecompat=1
 runkrazy=1
 runcppcheck=1
 runtidy=1
@@ -454,15 +469,16 @@ runsplint=1
 while true ; do
     case "$1" in
         -h|--help) HELP; exit 1;;
-        -k|--no-krazy)       runkrazy=0;      shift;;
-        -c|--no-cppcheck)    runcppcheck=0;   shift;;
-        -t|--no-tidy)        runtidy=0;       shift;;
-        -b|--no-scan)        runscan=0;       shift;;
-        -s|--no-splint)      runsplint=0;     shift;;
-        -l|--no-clang-build) runclangbuild=0; shift;;
-        -g|--no-gcc-build)   rungccbuild=0;   shift;;
-        -a|--no-asan-build)  runasanbuild=0;  shift;;
-        -d|--no-tsan-build)  runtsanbuild=0;  shift;;
+        -m|--no-cmake-compat) cmakecompat=0;   shift;;
+        -k|--no-krazy)        runkrazy=0;      shift;;
+        -c|--no-cppcheck)     runcppcheck=0;   shift;;
+        -t|--no-tidy)         runtidy=0;       shift;;
+        -b|--no-scan)         runscan=0;       shift;;
+        -s|--no-splint)       runsplint=0;     shift;;
+        -l|--no-clang-build)  runclangbuild=0; shift;;
+        -g|--no-gcc-build)    rungccbuild=0;   shift;;
+        -a|--no-asan-build)   runasanbuild=0;  shift;;
+        -d|--no-tsan-build)   runtsanbuild=0;  shift;;
         --) shift; break;;
         *)  echo "Internal error!"; exit 1;;
     esac
@@ -475,6 +491,27 @@ cd ..
 TOP=`pwd`
 BDIR=""
 
+#use minimum cmake version unless the --no-cmake-compat option is specified
+if ( test $cmakecompat -eq 1 )
+then
+  if ( test ! -e $TOP/CMakeLists.txt )
+  then
+    echo "Unable to locate the project top-level CMakeLists.txt.  Fix me"
+    exit 1
+  fi
+  # read the min required CMake version from the top-level CMake file
+  minCMakeVers=`grep -i cmake_minimum_required $TOP/CMakeLists.txt | grep VERSION | sed 's/^.*VERSION\s*//i' | cut -d. -f1-2 | sed 's/\s*).*$//'`
+  # adjust PATH
+  export PATH=/usr/local/opt/cmake-$minCMakeVers/bin:$PATH
+  # check the version
+  if ( test `cmake --version | head -1 | grep -c $minCMakeVers` -ne 1 )
+  then
+    echo "Not using cmake version $minCMakeVers"
+    echo "Maybe you need to install it into /usr/local/opt/cmake-$minCMakeVers"
+    exit 1
+  fi
+fi
+
 CMAKEOPTS="-DCMAKE_BUILD_TYPE=Debug -DGOBJECT_INTROSPECTION=False -DICAL_GLIB=False -DICAL_BUILD_DOCS=False"
 UUCCMAKEOPTS="$CMAKEOPTS -DCMAKE_DISABLE_FIND_PACKAGE_ICU=True"
 TZCMAKEOPTS="$CMAKEOPTS -DUSE_BUILTIN_TZDATA=True"
