@@ -17,7 +17,7 @@
 ========================================================================*/
 
 /**
-  @file icalrecur.c
+  icalrecur.c
   @brief Implementation of routines for dealing with recurring time
 
   How this code works:
@@ -145,9 +145,13 @@
 #if defined(HAVE_LIBICU)
 #include <unicode/ucal.h>
 #include <unicode/ustring.h>
-#define RSCALE_IS_SUPPORTED 1
+#if defined(HAVE_STDBOOL_H)
+#include <stdbool.h>
+#else
+#define false 0
+#define true 1
+#endif
 #else
-#define RSCALE_IS_SUPPORTED 0
 
 /* The maximums below are based on Gregorian leap years */
 #undef ICAL_BY_MONTH_SIZE
@@ -187,7 +191,7 @@
 
 int icalrecurrencetype_rscale_is_supported(void)
 {
-    return RSCALE_IS_SUPPORTED;
+    return 1;
 }
 
 /****************** Enumeration Routines ******************/
@@ -587,86 +591,148 @@ struct icalrecurrencetype icalrecurrencetype_from_string(const char *str)
                 break;
             }
         } else if (strcasecmp(name, "FREQ") == 0) {
-            parser.rt.freq = icalrecur_string_to_freq(value);
-            if (parser.rt.freq == ICAL_NO_RECURRENCE) {
+            if (parser.rt.freq != ICAL_NO_RECURRENCE) {
+                /* Don't allow multiple FREQs */
                 r = -1;
+            } else {
+                parser.rt.freq = icalrecur_string_to_freq(value);
+                if (parser.rt.freq == ICAL_NO_RECURRENCE) {
+                    r = -1;
+                }
             }
         } else if (strcasecmp(name, "RSCALE") == 0) {
-            if (icalrecurrencetype_rscale_is_supported()) {
-                parser.rt.rscale = icalmemory_strdup(value);
+            if (parser.rt.rscale != NULL) {
+                /* Don't allow multiple RSCALEs */
+                r = -1;
             } else {
-                r = -2; // RSCALE clause name without RSCALE support
+                parser.rt.rscale = icalmemory_strdup(value);
             }
         } else if (strcasecmp(name, "SKIP") == 0) {
-            if (icalrecurrencetype_rscale_is_supported()) {
+            if (parser.rt.skip != ICAL_SKIP_OMIT) {
+                /* Don't allow multiple SKIPs */
+                r = -1;
+            } else {
                 parser.rt.skip = icalrecur_string_to_skip(value);
                 if (parser.rt.skip == ICAL_SKIP_UNDEFINED) {
                     r = -1;
                 }
-            } else {
-                r = -2; // RSCALE clause name without RSCALE support
             }
         } else if (strcasecmp(name, "COUNT") == 0) {
-            parser.rt.count = atoi(value);
-            if (parser.rt.count < 1) {
-                /* don't allow count to be less than 1 */
-                r = -1;
-            } else if (!icaltime_is_null_time(parser.rt.until)) {
-                /* don't allow both count and until */
-                r = -1;
-            }
-        } else if (strcasecmp(name, "UNTIL") == 0) {
-            parser.rt.until = icaltime_from_string(value);
-            if (icaltime_is_null_time(parser.rt.until)) {
-                r = -1;
-            } else if (parser.rt.count > 0) {
-               /* don't allow both count and until */
-               r = -1;
-            }
-        } else if (strcasecmp(name, "INTERVAL") == 0) {
-            parser.rt.interval = (short)atoi(value);
-            /* don't allow an interval to be less than 1
-               (RFC specifies an interval must be a positive integer) */
-            if (parser.rt.interval < 1) r = -1;
-        } else if (strcasecmp(name, "WKST") == 0) {
-            parser.rt.week_start = icalrecur_string_to_weekday(value);
-            if (parser.rt.week_start == ICAL_NO_WEEKDAY) {
+            if (parser.rt.count > 0 || !icaltime_is_null_time(parser.rt.until)) {
+                /* Don't allow multiple COUNTs, or both COUNT and UNTIL */
                 r = -1;
             } else {
-                sort_bydayrules(&parser);
+                parser.rt.count = atoi(value);
+                /* don't allow count to be less than 1 */
+                if (parser.rt.count < 1) r = -1;
+            }
+        } else if (strcasecmp(name, "UNTIL") == 0) {
+            if (parser.rt.count > 0 || !icaltime_is_null_time(parser.rt.until)) {
+                /* Don't allow multiple COUNTs, or both COUNT and UNTIL */
+                r = -1;
+            } else {
+                parser.rt.until = icaltime_from_string(value);
+                if (icaltime_is_null_time(parser.rt.until)) r = -1;
+            }
+        } else if (strcasecmp(name, "INTERVAL") == 0) {
+            if (parser.rt.interval > 1) {
+                /* Don't allow multiple INTERVALs */
+                r = -1;
+            } else {
+                parser.rt.interval = (short)atoi(value);
+                /* don't allow an interval to be less than 1
+                   (RFC specifies an interval must be a positive integer) */
+                if (parser.rt.interval < 1) r = -1;
+            }
+        } else if (strcasecmp(name, "WKST") == 0) {
+            if (parser.rt.week_start != ICAL_MONDAY_WEEKDAY) {
+                /* Don't allow multiple WKSTs */
+                r = -1;
+            } else {
+                parser.rt.week_start = icalrecur_string_to_weekday(value);
+                if (parser.rt.week_start == ICAL_NO_WEEKDAY) {
+                    r = -1;
+                } else {
+                    sort_bydayrules(&parser);
+                }
             }
         } else if (strcasecmp(name, "BYSECOND") == 0) {
-            r = icalrecur_add_byrules(&parser, parser.rt.by_second,
-                                      0, ICAL_BY_SECOND_SIZE, value);
+            if (parser.rt.by_second[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYSECONDs */
+                r = -1;
+            } else {
+                r = icalrecur_add_byrules(&parser, parser.rt.by_second,
+                                          0, ICAL_BY_SECOND_SIZE, value);
+            }
         } else if (strcasecmp(name, "BYMINUTE") == 0) {
-            r = icalrecur_add_byrules(&parser, parser.rt.by_minute,
-                                      0, ICAL_BY_MINUTE_SIZE, value);
+            if (parser.rt.by_minute[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYMINUTEs */
+                r = -1;
+            } else {
+                r = icalrecur_add_byrules(&parser, parser.rt.by_minute,
+                                          0, ICAL_BY_MINUTE_SIZE, value);
+            }
         } else if (strcasecmp(name, "BYHOUR") == 0) {
-            r = icalrecur_add_byrules(&parser, parser.rt.by_hour,
-                                      0, ICAL_BY_HOUR_SIZE, value);
+            if (parser.rt.by_hour[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYHOURs */
+                r = -1;
+            } else {
+                r = icalrecur_add_byrules(&parser, parser.rt.by_hour,
+                                          0, ICAL_BY_HOUR_SIZE, value);
+            }
         } else if (strcasecmp(name, "BYDAY") == 0) {
-            r = icalrecur_add_bydayrules(&parser, value);
+            if (parser.rt.by_day[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYDAYs */
+                r = -1;
+            } else {
+                r = icalrecur_add_bydayrules(&parser, value);
+            }
         } else if (strcasecmp(name, "BYMONTHDAY") == 0) {
-            r = icalrecur_add_byrules(&parser, parser.rt.by_month_day,
-                                      -1, ICAL_BY_MONTHDAY_SIZE, value);
+            if (parser.rt.by_month_day[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYMONTHDAYs */
+                r = -1;
+            } else {
+                r = icalrecur_add_byrules(&parser, parser.rt.by_month_day,
+                                          -1, ICAL_BY_MONTHDAY_SIZE, value);
+            }
         } else if (strcasecmp(name, "BYYEARDAY") == 0) {
-            r = icalrecur_add_byrules(&parser, parser.rt.by_year_day,
-                                      -1, ICAL_BY_YEARDAY_SIZE, value);
+            if (parser.rt.by_year_day[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYYEARDAYs */
+                r = -1;
+            } else {
+                r = icalrecur_add_byrules(&parser, parser.rt.by_year_day,
+                                          -1, ICAL_BY_YEARDAY_SIZE, value);
+            }
         } else if (strcasecmp(name, "BYWEEKNO") == 0) {
-            r = icalrecur_add_byrules(&parser, parser.rt.by_week_no,
-                                      -1, ICAL_BY_WEEKNO_SIZE, value);
+            if (parser.rt.by_week_no[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYWEEKNOs */
+                r = -1;
+            } else {
+                r = icalrecur_add_byrules(&parser, parser.rt.by_week_no,
+                                          -1, ICAL_BY_WEEKNO_SIZE, value);
+            }
         } else if (strcasecmp(name, "BYMONTH") == 0) {
-            r = icalrecur_add_byrules(&parser, parser.rt.by_month,
-                                      1, ICAL_BY_MONTH_SIZE, value);
+            if (parser.rt.by_month[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYMONTHs */
+                r = -1;
+            } else {
+                r = icalrecur_add_byrules(&parser, parser.rt.by_month,
+                                          1, ICAL_BY_MONTH_SIZE, value);
+            }
         } else if (strcasecmp(name, "BYSETPOS") == 0) {
-            r = icalrecur_add_byrules(&parser, parser.rt.by_set_pos,
-                                      -1, ICAL_BY_SETPOS_SIZE, value);
+            if (parser.rt.by_set_pos[0] != ICAL_RECURRENCE_ARRAY_MAX) {
+                /* Don't allow multiple BYSETPOSs */
+                r = -1;
+            } else {
+                r = icalrecur_add_byrules(&parser, parser.rt.by_set_pos,
+                                          -1, ICAL_BY_SETPOS_SIZE, value);
+            }
         } else {
             r = -1;
         }
 
         if (r) {
-            /* Note: silently ignore when we have an RSCALE rule, yet don't have RSCALE support.
+            /* Note: silently ignore when we have a leap month, yet don't have RSCALE support.
                The magic value "-2" indicates when that happens.
             */
             if (r != -2) {
@@ -985,7 +1051,18 @@ static void setup_defaults(icalrecur_iterator *impl,
     }
 }
 
-/* Calculate the number of Gregorian months between 2 dates */
+/** Calculate ISO weeks per year
+   https://en.wikipedia.org/wiki/ISO_week_date#Weeks_per_year */
+static int weeks_in_year(int year)
+{
+    /* Long years occur when year starts on Thu or leap year starts on Wed */
+    int dow = icaltime_day_of_week(icaltime_from_day_of_year(1, year));
+    int is_long = (dow == 5 || (dow == 4 && icaltime_is_leap_year(year)));
+
+    return (52 + is_long);
+}
+
+/** Calculate the number of Gregorian months between 2 dates */
 static int __greg_month_diff(icaltimetype a, icaltimetype b)
 {
     return (12 * (b.year - a.year) + (b.month - a.month));
@@ -1018,7 +1095,7 @@ icalarray *icalrecurrencetype_rscale_supported_calendars(void)
 
     calendars = icalarray_new(sizeof(const char **), 20);
 
-    en = ucal_getKeywordValuesForLocale("calendar", NULL, FALSE, &status);
+    en = ucal_getKeywordValuesForLocale("calendar", NULL, false, &status);
     while ((cal = uenum_next(en, NULL, &status))) {
         cal = icalmemory_tmp_copy(cal);
         icalarray_append(calendars, &cal);
@@ -1121,8 +1198,8 @@ static int get_days_in_year(icalrecur_iterator *impl, int year)
 
 static void set_day_of_year(icalrecur_iterator *impl, int doy)
 {
-    if (doy < 0) {
-        doy += get_days_in_year(impl, 0) + 1;
+    if (doy < 1) {
+        doy += get_days_in_year(impl, 0);
     }
 
     ucal_set(impl->rscale, UCAL_DAY_OF_YEAR, (int32_t) doy);
@@ -1411,7 +1488,7 @@ static int initialize_rscale(icalrecur_iterator *impl)
         }
 
         /* Check if specified calendar is supported */
-        en = ucal_getKeywordValuesForLocale("calendar", NULL, FALSE, &status);
+        en = ucal_getKeywordValuesForLocale("calendar", NULL, false, &status);
         while ((cal = uenum_next(en, NULL, &status))) {
             if (!strcmp(cal, rule.rscale)) {
                 is_hebrew = !strcmp(rule.rscale, "hebrew");
@@ -1464,7 +1541,7 @@ static int initialize_rscale(icalrecur_iterator *impl)
     return 1;
 }
 
-/* Set Gregorian date and convert to RSCALE */
+/** Sets the Gregorian date and convert to RSCALE */
 static void set_datetime(icalrecur_iterator *impl, icaltimetype date)
 {
     UErrorCode status = U_ZERO_ERROR;
@@ -1497,7 +1574,7 @@ static void set_datetime(icalrecur_iterator *impl, icaltimetype date)
     }
 }
 
-/* Calculate the number of RSCALE months between 2 dates */
+/** Calculate the number of RSCALE months between 2 dates */
 static int month_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
 {
     int diff;
@@ -1528,7 +1605,7 @@ static int month_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
     return diff;
 }
 
-/* Calculate the number of RSCALE days between 2 dates */
+/** Calculate the number of RSCALE days between 2 dates */
 static int day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
 {
     UErrorCode status = U_ZERO_ERROR;
@@ -1548,6 +1625,13 @@ static int day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
     return diff;
 }
 
+static void reset_period_start(icalrecur_iterator *impl)
+{
+    struct icaltimetype start = impl->period_start;
+
+    (void)get_day_of_year(impl, start.year, start.month, start.day, NULL);
+}
+
 #else /* !HAVE_LIBICU */
 
 /*
@@ -1556,7 +1640,12 @@ static int day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
 
 icalarray *icalrecurrencetype_rscale_supported_calendars(void)
 {
-    return NULL;
+    icalarray *calendars = icalarray_new(sizeof(const char **), 1);
+    const char *cal = "gregorian";
+
+    icalarray_append(calendars, &cal);
+
+    return calendars;
 }
 
 static void set_second(icalrecur_iterator *impl, int second)
@@ -1592,8 +1681,8 @@ static void set_day_of_year(icalrecur_iterator *impl, int doy)
 {
     struct icaltimetype next;
 
-    if (doy < 0) {
-        doy += get_days_in_year(impl, impl->last.year) + 1;
+    if (doy < 1) {
+        doy += get_days_in_year(impl, impl->last.year);
     }
 
     next = icaltime_from_day_of_year(doy, impl->last.year);
@@ -1613,18 +1702,7 @@ static int get_day_of_week(icalrecur_iterator *impl)
     return icaltime_day_of_week(impl->last);
 }
 
-/* Calculate ISO weeks per year
-   https://en.wikipedia.org/wiki/ISO_week_date#Weeks_per_year */
-static int weeks_in_year(int year)
-{
-    /* Long years occur when year starts on Thu or leap year starts on Wed */
-    int dow = icaltime_day_of_week(icaltime_from_day_of_year(1, year));
-    int is_long = (dow == 5 || (dow == 4 && icaltime_is_leap_year(year)));
-
-    return (52 + is_long);
-}
-
-/* Calculate ISO week number
+/** Calculate ISO week number
    https://en.wikipedia.org/wiki/ISO_week_date#Calculation */
 static int get_week_number(icalrecur_iterator *impl, struct icaltimetype tt)
 {
@@ -1754,12 +1832,17 @@ static void increment_second(icalrecur_iterator *impl, int inc)
 
 static int initialize_rscale(icalrecur_iterator *impl)
 {
+    if (impl->rule.rscale && strcasecmp(impl->rule.rscale, "GREGORIAN")) {
+        icalerror_set_errno(ICAL_UNIMPLEMENTED_ERROR);
+        return 0;
+    }
+
     impl->rstart = impl->dtstart;
 
     return 1;
 }
 
-/* Set Gregorian date */
+/** Sets the Gregorian date */
 static void set_datetime(icalrecur_iterator *impl, icaltimetype date)
 {
     impl->last.year = date.year;
@@ -1778,7 +1861,7 @@ static void set_datetime(icalrecur_iterator *impl, icaltimetype date)
     }
 }
 
-/* Calculate the number of Gregorian months between 2 dates */
+/** Calculate the number of Gregorian months between 2 dates */
 static int month_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
 {
     _unused(impl);
@@ -1786,12 +1869,17 @@ static int month_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
     return __greg_month_diff(a, b);
 }
 
-/* Calculate the number of Gregorian days between 2 dates */
+/** Calculate the number of Gregorian days between 2 dates */
 static int day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
 {
     return __day_diff(impl, a, b);
 }
 
+static void reset_period_start(icalrecur_iterator *impl)
+{
+    set_datetime(impl, impl->period_start);
+}
+
 #endif /* HAVE_LIBICU */
 
 static int __iterator_set_start(icalrecur_iterator *impl, icaltimetype start);
@@ -1957,7 +2045,7 @@ void icalrecur_iterator_free(icalrecur_iterator *i)
     free(i);
 }
 
-/* Calculate the number of days between 2 dates */
+/** Calculate the number of days between 2 dates */
 static int __day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
 {
     int diff;
@@ -1995,7 +2083,7 @@ static int __day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
     return diff;
 }
 
-/** Increment month is different that the other increment_* routines --
+/** increment_month is different than the other increment_* routines --
    it figures out the interval for itself, and uses BYMONTH data if
    available. */
 static void increment_month(icalrecur_iterator *impl)
@@ -2053,154 +2141,76 @@ void test_increment()
 
 #endif
 
-static int next_second(icalrecur_iterator *impl)
+static int next_unit(icalrecur_iterator *impl,
+                     int by_unit, icalrecurrencetype_frequency frequency,
+                     int (*next_sub_unit)(icalrecur_iterator *),
+                     void (*set_unit)(icalrecur_iterator *, int),
+                     void (*increment_unit)(icalrecur_iterator *, int),
+                     void (*increment_super_unit)(icalrecur_iterator *, int))
 {
-    int has_by_second =
-        (impl->by_ptrs[BY_SECOND][0] != ICAL_RECURRENCE_ARRAY_MAX);
-    int this_frequency = (impl->rule.freq == ICAL_SECONDLY_RECURRENCE);
+    int has_by_unit = (by_unit > NO_CONTRACTION) &&
+        (impl->by_ptrs[by_unit][0] != ICAL_RECURRENCE_ARRAY_MAX);
+    int this_frequency = (impl->rule.freq == frequency);
 
     int end_of_data = 0;
 
-    assert(has_by_second || this_frequency);
+    assert(has_by_unit || this_frequency);
 
-    if (has_by_second) {
+    if (next_sub_unit && next_sub_unit(impl) == 0) {
+        return 0;
+    }
+
+    if (has_by_unit) {
         /* Ignore the frequency and use the byrule data */
 
-        impl->by_indices[BY_SECOND]++;
+        impl->by_indices[by_unit]++;
 
-        if (impl->by_ptrs[BY_SECOND][impl->by_indices[BY_SECOND]] ==
+        if (impl->by_ptrs[by_unit][impl->by_indices[by_unit]] ==
             ICAL_RECURRENCE_ARRAY_MAX) {
-            impl->by_indices[BY_SECOND] = 0;
+            impl->by_indices[by_unit] = 0;
 
             end_of_data = 1;
         }
 
-        set_second(impl, impl->by_ptrs[BY_SECOND][impl->by_indices[BY_SECOND]]);
+        set_unit(impl, impl->by_ptrs[by_unit][impl->by_indices[by_unit]]);
 
-    } else if (!has_by_second && this_frequency) {
+    } else if (!has_by_unit && this_frequency) {
         /* Compute the next value from the last time and the freq interval */
-        increment_second(impl, impl->rule.interval);
+        increment_unit(impl, impl->rule.interval);
     }
 
-    /* If we have gone through all of the seconds on the BY list, then we
-       need to move to the next minute */
+    /* If we have gone through all of the units on the BY list, then we
+       need to move to the next larger unit */
 
-    if (has_by_second && end_of_data && this_frequency) {
-        increment_minute(impl, 1);
+    if (has_by_unit && end_of_data && this_frequency) {
+        increment_super_unit(impl, 1);
     }
 
     return end_of_data;
 }
 
+static int next_second(icalrecur_iterator *impl)
+{
+    return next_unit(impl, BY_SECOND, ICAL_SECONDLY_RECURRENCE, NULL,
+                     &set_second, &increment_second, &increment_minute);
+}
+
 static int next_minute(icalrecur_iterator *impl)
 {
-    int has_by_minute =
-        (impl->by_ptrs[BY_MINUTE][0] != ICAL_RECURRENCE_ARRAY_MAX);
-    int this_frequency = (impl->rule.freq == ICAL_MINUTELY_RECURRENCE);
-
-    int end_of_data = 0;
-
-    assert(has_by_minute || this_frequency);
-
-    if (next_second(impl) == 0) {
-        return 0;
-    }
-
-    if (has_by_minute) {
-        /* Ignore the frequency and use the byrule data */
-
-        impl->by_indices[BY_MINUTE]++;
-
-        if (impl->by_ptrs[BY_MINUTE][impl->by_indices[BY_MINUTE]]
-            == ICAL_RECURRENCE_ARRAY_MAX) {
-
-            impl->by_indices[BY_MINUTE] = 0;
-
-            end_of_data = 1;
-        }
-
-        set_minute(impl, impl->by_ptrs[BY_MINUTE][impl->by_indices[BY_MINUTE]]);
-
-    } else if (!has_by_minute && this_frequency) {
-        /* Compute the next value from the last time and the freq interval */
-        increment_minute(impl, impl->rule.interval);
-    }
-
-    /* If we have gone through all of the minutes on the BY list, then we
-       need to move to the next hour */
-
-    if (has_by_minute && end_of_data && this_frequency) {
-        increment_hour(impl, 1);
-    }
-
-    return end_of_data;
+    return next_unit(impl, BY_MINUTE, ICAL_MINUTELY_RECURRENCE, &next_second,
+                     &set_minute, &increment_minute, &increment_hour);
 }
 
 static int next_hour(icalrecur_iterator *impl)
 {
-    int has_by_hour = (impl->by_ptrs[BY_HOUR][0] != ICAL_RECURRENCE_ARRAY_MAX);
-    int this_frequency = (impl->rule.freq == ICAL_HOURLY_RECURRENCE);
-
-    int end_of_data = 0;
-
-    assert(has_by_hour || this_frequency);
-
-    if (next_minute(impl) == 0) {
-        return 0;
-    }
-
-    if (has_by_hour) {
-        /* Ignore the frequency and use the byrule data */
-
-        impl->by_indices[BY_HOUR]++;
-
-        if (impl->by_ptrs[BY_HOUR][impl->by_indices[BY_HOUR]] ==
-            ICAL_RECURRENCE_ARRAY_MAX) {
-            impl->by_indices[BY_HOUR] = 0;
-
-            end_of_data = 1;
-        }
-
-        set_hour(impl, impl->by_ptrs[BY_HOUR][impl->by_indices[BY_HOUR]]);
-
-    } else if (!has_by_hour && this_frequency) {
-        /* Compute the next value from the last time and the freq interval */
-        increment_hour(impl, impl->rule.interval);
-    }
-
-    /* If we have gone through all of the hours on the BY list, then we
-       need to move to the next day */
-
-    if (has_by_hour && end_of_data && this_frequency) {
-        increment_monthday(impl, 1);
-    }
-
-    return end_of_data;
+    return next_unit(impl, BY_HOUR, ICAL_HOURLY_RECURRENCE, &next_minute,
+                     &set_hour, &increment_hour, &increment_monthday);
 }
 
 static int next_day(icalrecur_iterator *impl)
 {
-    int has_by_day = (impl->by_ptrs[BY_DAY][0] != ICAL_RECURRENCE_ARRAY_MAX);
-    int this_frequency = (impl->rule.freq == ICAL_DAILY_RECURRENCE);
-
-    assert(has_by_day || this_frequency);
-    _unused(has_by_day);
-
-    if (next_hour(impl) == 0) {
-        return 0;
-    }
-
-    /* Always increment through the interval, since this routine is not
-       called by any other next_* routine, and the days that are
-       excluded will be taken care of by restriction filtering */
-
-    if (this_frequency) {
-        increment_monthday(impl, impl->rule.interval);
-    } else {
-        increment_monthday(impl, 1);
-    }
-
-    return 0;
+    return next_unit(impl, NO_CONTRACTION, ICAL_DAILY_RECURRENCE, &next_hour,
+                     NULL, &increment_monthday, NULL);
 }
 
 static int check_set_position(icalrecur_iterator *impl, int set_pos)
@@ -2220,14 +2230,14 @@ static int check_set_position(icalrecur_iterator *impl, int set_pos)
     return found;
 }
 
-/* Add each BYMONTHDAY to the year days bitmask */
+/** Add each BYMONTHDAY to the year days bitmask */
 static int expand_bymonth_days(icalrecur_iterator *impl, int year, int month)
 {
     int i, set_pos_total = 0;
     int days_in_month = get_days_in_month(impl, month, year);
 
     for (i = 0; BYMDPTR[i] != ICAL_RECURRENCE_ARRAY_MAX; i++) {
-        short doy = 0, mday = BYMDPTR[i];
+        short doy = ICAL_BY_YEARDAY_SIZE, mday = BYMDPTR[i];
         int this_month = month;
 
         if (abs(mday) > days_in_month) {
@@ -2256,7 +2266,7 @@ static int expand_bymonth_days(icalrecur_iterator *impl, int year, int month)
                 }
 
                 if (this_month == 0) {
-                    doy = -1;      /* Last day of prev year */
+                    doy = 0;       /* Last day of prev year */
                 } else {
                     mday = -1;     /* Last day of month */
                 }
@@ -2264,7 +2274,7 @@ static int expand_bymonth_days(icalrecur_iterator *impl, int year, int month)
             }
         }
 
-        if (!doy) {
+        if (doy == ICAL_BY_YEARDAY_SIZE) {
             doy = get_day_of_year(impl, year, this_month, mday, NULL);
         }
 
@@ -2276,7 +2286,7 @@ static int expand_bymonth_days(icalrecur_iterator *impl, int year, int month)
     return set_pos_total;
 }
 
-/* Expand the BYDAY rule part and apply it to the year days map. */
+/** Expand the BYDAY rule part and apply it to the year days map. */
 static int expand_by_day(icalrecur_iterator *impl, int year,
                          int doy_offset, int last_day,
                          int first_dow, int last_dow,
@@ -2323,22 +2333,34 @@ static int expand_by_day(icalrecur_iterator *impl, int year,
             }
         }
 
-        (void)__icaltime_from_day_of_year(impl, day + doy_offset, year,
-                                          &this_weekno);
+        if (doy_offset < 0) {
+            this_weekno = 1;
+        } else {
+            (void)__icaltime_from_day_of_year(impl, day + doy_offset, year,
+                                              &this_weekno);
+        }
 
         /* Add instance(s) of the weekday within the period */
         do {
-            int valid = 1;
+            int valid = 0;
 
             if (has_by_data(impl, BY_WEEK_NO)) {
                 /* Make sure our day falls in one of the BYWEEKNO */
+                int nweeks = weeks_in_year(year);
                 int j;
 
                 for (j = 0; BYWEEKPTR[j] != ICAL_RECURRENCE_ARRAY_MAX; j++) {
                     int weekno = BYWEEKPTR[j];
 
-                    if (weekno != this_weekno) valid = 0;
+                    if (weekno < 0) weekno += nweeks + 1;
+
+                    if (weekno == this_weekno) {
+                        valid = 1;
+                        break;
+                    }
                 }
+            } else {
+                valid = 1;
             }
 
             if (valid) {
@@ -2372,7 +2394,7 @@ static int expand_by_day(icalrecur_iterator *impl, int year,
     return set_pos_total;
 }
 
-/* "Filter" the year days bitmask with each BYSETPOS */
+/** "Filter" the year days bitmask with each BYSETPOS */
 static void filter_bysetpos(icalrecur_iterator *impl, int pos_total,
                             int start_doy, int end_doy)
 {
@@ -2396,7 +2418,7 @@ static void filter_bysetpos(icalrecur_iterator *impl, int pos_total,
     }
 }
 
-/* For INTERVAL=MONTHLY, set up the year days bitmask in the iterator to
+/** For INTERVAL=MONTHLY, set up the year days bitmask in the iterator to
    list all of the days of the current month that are specified in this
    rule. */
 static int expand_month_days(icalrecur_iterator *impl, int year, int month)
@@ -2476,7 +2498,7 @@ static int next_weekday_by_week(icalrecur_iterator *impl)
             end_of_data = 1;    /* Signal that we're at the end */
         }
 
-        /* Add the day of week offset to to the start of this week, and use
+        /* Add the day of week offset to the start of this week, and use
            that to get the next day */
         /* ignore position of dow ("4FR"), only use dow ("FR") */
         dow = icalrecurrencetype_day_day_of_week(BYDAYPTR[BYDAYIDX]);
@@ -2494,7 +2516,6 @@ static int next_weekday_by_week(icalrecur_iterator *impl)
             }
 
             increment_year(impl, -1);
-            start_of_week--;    /* set_day_of_year() assumes last doy == -1 */
         }
 
         set_day_of_year(impl, start_of_week + dow);
@@ -2522,7 +2543,7 @@ static int next_week(icalrecur_iterator *impl)
     return end_of_data;
 }
 
-/* For INTERVAL=YEARLY, set up the year days bitmask in the iterator to
+/** For INTERVAL=YEARLY, set up the year days bitmask in the iterator to
    list all of the days of the current year that are specified in this
    rule. */
 static int expand_year_days(icalrecur_iterator *impl, int year)
@@ -2569,7 +2590,7 @@ static int expand_year_days(icalrecur_iterator *impl, int year)
 
                 case ICAL_SKIP_BACKWARD:
                     if (doy < 0) {
-                        doy = -1;                /* Last day of prev year */
+                        doy = 0;                 /* Last day of prev year */
                     } else {
                         doy = days_in_year;      /* Last day of this year */
                     }
@@ -2595,22 +2616,31 @@ static int expand_year_days(icalrecur_iterator *impl, int year)
             return 0;
         }
 
-        /* Calculate location of DTSTART day in weekno 1 */
-        doy = get_day_of_year(impl, year,
-                              impl->dtstart.month, impl->dtstart.day, NULL);
-        (void)__icaltime_from_day_of_year(impl, doy, year, &weekno);
-        if (weekno > doy) weekno = 0;
-        start_doy = doy - 7 * (weekno - 1);
+        /* BYWEEKNO + BYDAY handled below */
+        if (!has_by_data(impl, BY_DAY)) {
+            int nweeks = weeks_in_year(year);
 
-        /* Add day of week in each BYWEEKNO to the year days bitmask */
-        for (i = 0; BYWEEKPTR[i] != ICAL_RECURRENCE_ARRAY_MAX; i++) {
-            weekno = BYWEEKPTR[i];
+            /* Calculate location of DTSTART day in weekno 1 */
+            doy = get_day_of_year(impl, year,
+                                  impl->dtstart.month, impl->dtstart.day, NULL);
+            (void)__icaltime_from_day_of_year(impl, doy, year, &weekno);
+            if (weekno > doy) doy += 7;
+            start_doy = doy + get_start_of_week(impl);
 
-            doy = start_doy + 7 * (weekno - 1);
+            /* Add day of week in each BYWEEKNO to the year days bitmask */
+            for (i = 0; BYWEEKPTR[i] != ICAL_RECURRENCE_ARRAY_MAX; i++) {
+                weekno = BYWEEKPTR[i];
 
-            daysmask_setbit(impl->days, doy, 1);
-            set_pos_total++;
-            if (doy < impl->days_index) impl->days_index = doy;
+                if (weekno < 0) {
+                    weekno += nweeks + 1;
+                }
+
+                doy = start_doy + 7 * (weekno - 1);
+
+                daysmask_setbit(impl->days, doy, 1);
+                set_pos_total++;
+                if (doy < impl->days_index) impl->days_index = doy;
+            }
         }
     } else {
         /* Add each BYMONTHDAY in each BYMONTH to the year days bitmask */
@@ -2652,15 +2682,36 @@ static int expand_year_days(icalrecur_iterator *impl, int year)
             }
         } else {
             /* Numeric BYDAY are within the year */
+            short doy_offset = 0, last_day;
 
-            /* Get day of week of first day of year */
-            (void)get_day_of_year(impl, year, 1, 1, &first_dow);
+            if (has_by_data(impl, BY_WEEK_NO)) {
+                int weekno;
 
-            /* Get day of week of last day of year */
-            set_day_of_year(impl, days_in_year);
-            last_dow = get_day_of_week(impl);
+                /* See which week contains Jan 1 */
+                (void)__icaltime_from_day_of_year(impl, 1, year, &weekno);
+                if (weekno > 1) {
+                    /* Jan 1 is in last week of previous year - jump ahead */
+                    doy_offset += 7;
+                }
 
-            set_pos_total += expand_by_day(impl, year, 0, days_in_year,
+                /* Set start and end of ISO week-numbering year */
+                doy_offset += get_start_of_week(impl) - 1;
+                last_day = (7 * weeks_in_year(year)) - doy_offset - 1;
+
+                first_dow = impl->rule.week_start;
+                last_dow = (first_dow + 6) % 7;
+            } else {
+                /* Get day of week of first day of year */
+                (void)get_day_of_year(impl, year, 1, 1, &first_dow);
+
+                /* Get day of week of last day of year */
+                set_day_of_year(impl, days_in_year);
+                last_dow = get_day_of_week(impl);
+
+                last_day = days_in_year;
+            }
+
+            set_pos_total += expand_by_day(impl, year, doy_offset, last_day,
                                            first_dow, last_dow, limiting);
         }
     }
@@ -2688,22 +2739,76 @@ static int next_year(icalrecur_iterator *impl)
     return next_yearday(impl, &__next_year);
 }
 
+static short daymask_find_next_bit(icalrecur_iterator *impl)
+{
+    unsigned long *days = impl->days;
+    short days_index = impl->days_index + 1;
+    unsigned long v;
+    short startBitIndex;
+    int wordIdx;
+
+    if (days_index >= ICAL_YEARDAYS_MASK_SIZE)
+        return ICAL_YEARDAYS_MASK_SIZE;
+
+    // Prepare the first word, where searching might not start at the beginning
+    startBitIndex = days_index + ICAL_YEARDAYS_MASK_OFFSET;
+    wordIdx = (int)(startBitIndex / BITS_PER_LONG);
+    v = days[wordIdx];
+    v >>= startBitIndex % BITS_PER_LONG;
+
+    if (!v) {
+        // so the first word didn't contain any bits of interest.
+        days_index += BITS_PER_LONG - startBitIndex % BITS_PER_LONG;
+
+        // Are there more empty words following? Skip them.
+        while (days_index < ICAL_YEARDAYS_MASK_SIZE) {
+
+            wordIdx++;
+            v = days[wordIdx];
+
+            if (v)
+                break;
+
+            days_index += BITS_PER_LONG;
+        }
+    }
+
+    if (v) {
+
+        // We found a word containing the next bit but don't know the exact
+        // position yet. Do a b-search to find it.
+
+        unsigned long mask;
+        int maskSize = (int)(BITS_PER_LONG / 2);
+        mask = (((unsigned long)1) << maskSize) - 1;
+
+        while (maskSize) {
+
+            if ((v & mask) == 0) {
+                v >>= maskSize;
+                days_index += maskSize;
+            }
+            maskSize /= 2;
+            mask >>= maskSize;
+        }
+    }
+
+    return days_index;
+}
+
 static int next_yearday(icalrecur_iterator *impl,
                         void (*next_period)(icalrecur_iterator *))
 {
-    struct icaltimetype start = impl->period_start;
-
     if (next_hour(impl) == 0) {
         return 0;
     }
 
     /* We may have skipped fwd/bwd a month/year with previous occurrence.
        Reset the period start date so we can increment properly */
-    (void)get_day_of_year(impl, start.year, start.month, start.day, NULL);
+    reset_period_start(impl);
 
     /* Find next year day that is set */
-    while (++impl->days_index < ICAL_YEARDAYS_MASK_SIZE &&
-           !daysmask_getbit(impl->days, impl->days_index));
+    impl->days_index = daymask_find_next_bit(impl);
 
     if (impl->days_index >= ICAL_YEARDAYS_MASK_SIZE) {
 
@@ -2717,7 +2822,7 @@ static int next_yearday(icalrecur_iterator *impl,
         }
     }
 
-    if (impl->days_index < 0) {
+    if (impl->days_index < 1) {
         /* Day is in previous year */
         increment_year(impl, -1);
     }
@@ -3036,23 +3141,12 @@ void icalrecurrencetype_clear(struct icalrecurrencetype *recur)
     recur->week_start = ICAL_MONDAY_WEEKDAY;
     recur->freq = ICAL_NO_RECURRENCE;
     recur->interval = 1;
-    memset(&(recur->until), 0, sizeof(struct icaltimetype));
+    recur->until = icaltime_null_time();
     recur->count = 0;
     recur->rscale = NULL;
     recur->skip = ICAL_SKIP_OMIT;
 }
 
-/** The 'day' element of icalrecurrencetype_weekday is encoded to
- * allow representation of both the day of the week ( Monday, Tuesday),
- * but also the Nth day of the week ( First tuesday of the month, last
- * thursday of the year) These routines decode the day values.
- *
- * The day's position in the period ( Nth-ness) and the numerical
- * value of the day are encoded together as: pos*7 + dow
- *
- * A position of 0 means 'any' or 'every'
- */
-
 enum icalrecurrencetype_weekday icalrecurrencetype_day_day_of_week(short day)
 {
     return abs(day) % 8;
@@ -3069,12 +3163,12 @@ int icalrecurrencetype_day_position(short day)
     return pos;
 }
 
-/**
+/*
  * The 'month' element of the by_month array is encoded to allow
  * representation of the "L" leap suffix (RFC 7529).
  * These routines decode the month values.
  *
- * The "L" suffix is encoded by setting a high-order bit
+ * The "L" suffix is encoded by setting a high-order bit.
  */
 
 int icalrecurrencetype_month_is_leap(short month)
@@ -3087,12 +3181,6 @@ int icalrecurrencetype_month_month(short month)
     return (month & ~LEAP_MONTH);
 }
 
-/** Fill an array with the 'count' number of occurrences generated by
- * the rrule. Note that the times are returned in UTC, but the times
- * are calculated in local time. YOu will have to convert the results
- * back into local time before using them.
- */
-
 int icalrecur_expand_recurrence(const char *rule,
                                 time_t start, int count, time_t *array)
 {
