@@ -390,6 +390,16 @@ const struct recur rfc5545[] = {
      "FREQ=WEEKLY;BYDAY=WE,FR;INTERVAL=2;COUNT=4",
      NULL},
 
+    /* First 2 and last 2 ISO weeks of the year on Tue */
+    {"20130101T000000",
+     "FREQ=YEARLY;BYWEEKNO=1,2,-1,-2;BYDAY=TU;UNTIL=20170101T000000Z",
+     NULL},
+
+    /* 53rd ISO week of the year on Tue and Sat */
+    {"20130101T000000",
+     "FREQ=YEARLY;BYWEEKNO=53;BYDAY=TU,SA;UNTIL=20170101T000000Z",
+     NULL},
+
     {NULL, NULL, NULL}
 };
 
@@ -400,6 +410,7 @@ const struct recur rscale[] = {
      "RSCALE=RUSSIAN;FREQ=YEARLY",
      NULL},
 
+#if defined(HAVE_LIBICU)
     /* Ethiopic last day of year */
     {"20140910",
      "RSCALE=ETHIOPIC;FREQ=YEARLY;BYMONTH=13;BYMONTHDAY=-1;COUNT=6",
@@ -459,6 +470,11 @@ const struct recur rscale[] = {
      "RSCALE=CHINESE;FREQ=YEARLY;BYMONTHDAY=10;BYMONTH=9,9L;SKIP=BACKWARD;COUNT=4",
      NULL},
 
+    /* Every other Chinese New Year, starting at Jan 1, 2016 */
+    {"20130210",
+     "RSCALE=CHINESE;FREQ=YEARLY;UNTIL=20200101;INTERVAL=2",
+     "20160101"},
+
     /* Jewish leap month (omit) */
     {"20140205",
      "RSCALE=HEBREW;FREQ=YEARLY;COUNT=4",
@@ -493,6 +509,7 @@ const struct recur rscale[] = {
     {"20140302",
      "RSCALE=HEBREW;FREQ=YEARLY;BYMONTH=5L;BYMONTHDAY=-1;SKIP=BACKWARD;COUNT=5",
      NULL},
+#endif /* HAVE_LIBICU */
 
     /* Gregorian leap day (omit) */
     {"20120229",
@@ -559,18 +576,11 @@ const struct recur rscale[] = {
      "RSCALE=GREGORIAN;FREQ=YEARLY;BYYEARDAY=-366;SKIP=BACKWARD;COUNT=9",
      NULL},
 
-    /* Every other Chinese New Year, starting at Jan 1, 2016 */
-    {"20130210",
-     "RSCALE=CHINESE;FREQ=YEARLY;UNTIL=20200101;INTERVAL=2",
-     "20160101"},
-
     {NULL, NULL, NULL}
 };
 
 int main(int argc, char *argv[])
 {
-    int opt;
-
     /* Default to RFC 5545 tests */
     const struct recur *r = rfc5545;
     FILE *fp = fopen("test.out", "w");
@@ -581,29 +591,21 @@ int main(int argc, char *argv[])
         return (1);
     }
 
-    while ((opt = getopt(argc, argv, "rv")) != EOF) {
-        switch (opt) {
-#if defined(HAVE_LIBICU)
-        case 'r':      /* Do RSCALE tests */
-            if (!icalrecurrencetype_rscale_is_supported()) {
-                fprintf(stderr, "error: RSCALE not supported\n");
-                fclose(fp);
-                return (1);
-            }
+    /* Do not use getopt for command line parsing -- for portability on Windows */
+    for (int i = 1; i < argc; ++i) {
+        if (strncmp(argv[i], "-r", 2) == 0) { /* Do RSCALE tests */
             r = rscale;
-            break;
-#endif
-
-        case 'v':      /* Verbose output to stdout */
-            verbose = 1;
-            break;
-
-        default:
-            fprintf(stderr, "usage: %s [-r]\n", argv[0]);
-            fclose(fp);
-            return (1);
-            break;
+            continue;
         }
+
+        if (strncmp(argv[i], "-v", 2) == 0) { /* Verbose output to stdout */
+            verbose = 1;
+            continue;
+        }
+
+        fprintf(stderr, "usage: %s [-r]\n", argv[0]);
+        fclose(fp);
+        return (1);
     }
 
     for (; r->dtstart; r++) {
